searchData={"items":[{"type":"module","title":"Ecto.ULID","doc":"An Ecto type for ULID strings.","ref":"Ecto.ULID.html"},{"type":"function","title":"Ecto.ULID.bingenerate/1","doc":"Generates a binary ULID.\n\nOptionally takes a Unix timestamp with millisecond precision as an argument.\nIf no argument is passed, the current time is used as a base for the ULID.","ref":"Ecto.ULID.html#bingenerate/1"},{"type":"function","title":"Examples - Ecto.ULID.bingenerate/1","doc":"iex> Ecto.ULID.bingenerate()\n     >\n\n    iex> Ecto.ULID.bingenerate(1402899630000)\n     >","ref":"Ecto.ULID.html#bingenerate/1-examples"},{"type":"function","title":"Ecto.ULID.cast/1","doc":"Casts a string to ULID.","ref":"Ecto.ULID.html#cast/1"},{"type":"function","title":"Example - Ecto.ULID.cast/1","doc":"iex> Ecto.ULID.cast(\"018THNB1XG1KDQ084XT5H9GDS5\")\n    {:ok, \"018THNB1XG1KDQ084XT5H9GDS5\"}\n\n    iex> Ecto.ULID.cast(\"abc\")\n    :error","ref":"Ecto.ULID.html#cast/1-example"},{"type":"function","title":"Ecto.ULID.cast!/1","doc":"Same as `cast/1`, but raises `Ecto.CastError` on invalid arguments.","ref":"Ecto.ULID.html#cast!/1"},{"type":"function","title":"Example - Ecto.ULID.cast!/1","doc":"iex> Ecto.ULID.cast!(\"018THNB1XG1KDQ084XT5H9GDS5\")\n    \"018THNB1XG1KDQ084XT5H9GDS5\"\n\n    iex> Ecto.ULID.cast!(\"abc\")\n    ** (Ecto.CastError) cannot cast \"abc\" to Ecto.ULID","ref":"Ecto.ULID.html#cast!/1-example"},{"type":"function","title":"Ecto.ULID.dump/1","doc":"Converts a Crockford Base32 encoded ULID into a binary.","ref":"Ecto.ULID.html#dump/1"},{"type":"function","title":"Example - Ecto.ULID.dump/1","doc":"iex> Ecto.ULID.dump(\"018THNB1XG1KDQ084XT5H9GDS5\")\n    {:ok,  >}","ref":"Ecto.ULID.html#dump/1-example"},{"type":"function","title":"Ecto.ULID.embed_as/1","doc":"","ref":"Ecto.ULID.html#embed_as/1"},{"type":"function","title":"Ecto.ULID.equal?/2","doc":"","ref":"Ecto.ULID.html#equal?/2"},{"type":"function","title":"Ecto.ULID.generate/1","doc":"Generates a Crockford Base32 encoded ULID.\n\nOptionally takes a Unix timestamp with millisecond precision as an argument.\nIf no argument is passed, the current time is used as a base for the ULID.","ref":"Ecto.ULID.html#generate/1"},{"type":"function","title":"Examples - Ecto.ULID.generate/1","doc":"iex> Ecto.ULID.generate()\n    \"01FWEPCVGGMQQ0YTKS7BYFQH46\"\n\n    iex> Ecto.ULID.generate(1402899630000)\n    \"018THNB1XGQZ7T929PD126SM3Z\"","ref":"Ecto.ULID.html#generate/1-examples"},{"type":"function","title":"Ecto.ULID.load/1","doc":"Converts a binary ULID into a Crockford Base32 encoded string.","ref":"Ecto.ULID.html#load/1"},{"type":"function","title":"Example - Ecto.ULID.load/1","doc":"iex> Ecto.ULID.load( >)\n    {:ok, \"018THNB1XG1KDQ084XT5H9GDS5\"}","ref":"Ecto.ULID.html#load/1-example"},{"type":"function","title":"Ecto.ULID.type/0","doc":"The underlying schema type (`:uuid`).","ref":"Ecto.ULID.html#type/0"},{"type":"type","title":"Ecto.ULID.t/0","doc":"A hex-encoded ULID string.","ref":"Ecto.ULID.html#t:t/0"},{"type":"extras","title":"Ecto.ULID Next","doc":"# Ecto.ULID Next\n\n[![Elixir CI](https://github.com/woylie/ecto_ulid/actions/workflows/elixir.yml/badge.svg)](https://github.com/woylie/ecto_ulid/actions/workflows/elixir.yml)\n[![Hex](https://img.shields.io/hexpm/v/ecto_ulid_next)](https://hex.pm/packages/ecto_ulid_next)\n\nEcto.ULID Next is an `Ecto.Type` implementation for [ULID](https://github.com/ulid/spec).\n\nOriginally forked from [TheRealReal/ecto-ulid](https://github.com/TheRealReal/ecto-ulid),\nthis library is actively maintained to ensure compatibility with both current\nand upcoming Ecto releases.","ref":"readme.html"},{"type":"extras","title":"Features - Ecto.ULID Next","doc":"- Generate ULIDs in either Base32 or binary format.\n- Generate ULIDs based on a specific timestamp.\n- Autogenerate ULIDs when used as a primary key in your schema.\n- Read and write ULIDs using the database's native `uuid` type—no need for\n  additional database extensions.\n- Supports Ecto 3.2 and above.\n- Supports the [officially supported Elixir versions](https://hexdocs.pm/elixir/compatibility-and-deprecations.html) (currently ~> 1.11).\n- Tested and confirmed to work on both PostgreSQL and MySQL.\n- Optimized for high throughput scenarios.","ref":"readme.html#features"},{"type":"extras","title":"Why Choose ULID? - Ecto.ULID Next","doc":"ULID (Universally Unique Lexicographically Sortable Identifier) offers\nlexicographic sorting, unlike UUID. This feature simplifies chronological\nsorting and range queries, making ULID advantageous for time-series data, event\nsourcing, and other specific use-cases.\n\nNote: UUID v7 is a new proposed format that also aims to offer lexicographic\nsorting.","ref":"readme.html#why-choose-ulid"},{"type":"extras","title":"Compatibility Considerations - Ecto.ULID Next","doc":"ULID is binary-compatible with UUID, which means `Ecto.ULID` can be used\ninterchangeably in environments where `Ecto.UUID` is supported. It has been\nconfirmed to work with PostgreSQL and MySQL with the `uuid` column type.","ref":"readme.html#compatibility-considerations"},{"type":"extras","title":"Performance - Ecto.ULID Next","doc":"`Ecto.ULID` is optimized for quick operations, particularly useful in scenarios\nrequiring the handling of a large volume of events. The library borrows\ntechniques from `Ecto.UUID` to achieve sub-microsecond times for most\noperations.\n\nA benchmark suite is included to test the performance. Clone the repository and\nrun `mix run bench/ulid_bench.exs` to gauge how the library performs on your own\nsystem.","ref":"readme.html#performance"},{"type":"extras","title":"Installation - Ecto.ULID Next","doc":"To install `ecto_ulid_next`, add it to your project's dependencies in `mix.exs`:\n\n```elixir\ndefp deps do\n  [\n    {:ecto_ulid_next, \"~> 1.0.2\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage Scenarios - Ecto.ULID Next","doc":"The following sections outline three practical examples of how to use\n`Ecto.ULID`. For API details, refer to the\n[documentation](https://hexdocs.pm/ecto_ulid_next).\n\n#","ref":"readme.html#usage-scenarios"},{"type":"extras","title":"Using `Ecto.ULID` as a Primary Key or Foreign Key for a Single Table - Ecto.ULID Next","doc":"##","ref":"readme.html#using-ecto-ulid-as-a-primary-key-or-foreign-key-for-a-single-table"},{"type":"extras","title":"Migration - Ecto.ULID Next","doc":"In your migration file, specify `:binary_id` as the column type for the\nprimary key and any foreign keys.\n\n```elixir\ncreate table(:events, primary_key: false) do\n  add :id, :binary_id, primary_key: true\n  add :user_id, references(:users, type: :binary_id)\n  # additional fields\nend\n```\n\n##","ref":"readme.html#migration"},{"type":"extras","title":"Schema Definition - Ecto.ULID Next","doc":"In your schema, set both `@primary_key` and `@foreign_key_type` to `Ecto.ULID`.\nThis will default both the primary key and all foreign keys in the schema to use\nthe ULID type.\n\n```elixir\ndefmodule MyApp.Event do\n  use Ecto.Schema\n\n  @primary_key {:id, Ecto.ULID, autogenerate: true}\n  @foreign_key_type Ecto.ULID\n\n  schema \"events\" do\n    belongs_to :user, MyApp.User\n    # additional fields\n  end\nend\n```\n\nTo specify the ULID type for an individual association within the current\nschema, you can set the `type` option in the `belongs_to` function:\n\n```elixir\ndefmodule MyApp.Event do\n  use Ecto.Schema\n\n  @primary_key {:id, Ecto.ULID, autogenerate: true}\n\n  schema \"events\" do\n    belongs_to :user, MyApp.User, type: Ecto.ULID\n    # additional fields\n  end\nend\n```\n\n#","ref":"readme.html#schema-definition"},{"type":"extras","title":"Using `Ecto.ULID` as Primary Keys Across Multiple Tables - Ecto.ULID Next","doc":"##","ref":"readme.html#using-ecto-ulid-as-primary-keys-across-multiple-tables"},{"type":"extras","title":"Global Migration Configuration - Ecto.ULID Next","doc":"First, set a global configuration for all migrations to use `:binary_id` as the\nprimary key type in `config/config.exs`.\n\n```elixir\nconfig :my_app, MyApp.Repo,\n  migration_primary_key: [name: :id, type: :binary_id],\n  migration_foreign_key: [type: :binary_id]\n```\n\nWith this configuration, there's no need to explicitly define the primary key\ncolumn and the foreign key column type.\n\n```elixir\ncreate table(:events) do\n  belongs_to :user, MyApp.User\n  # additional fields\nend\n```\n\n##","ref":"readme.html#global-migration-configuration"},{"type":"extras","title":"Shared Schema Configuration - Ecto.ULID Next","doc":"Create a shared module to house general schema configurations. This module will\nset `Ecto.ULID` as the default for both primary and foreign keys.\n\n```elixir\ndefmodule MyApp.Schema do\n  @moduledoc false\n\n  defmacro __using__(_) do\n    quote do\n      use Ecto.Schema\n\n      @primary_key {:id, Ecto.ULID, autogenerate: true}\n      @foreign_key_type Ecto.ULID\n    end\n  end\nend\n```\n\nWhen defining individual schemas, simply use `MyApp.Schema` instead of\n`Ecto.Schema`. This eliminates the need to specify these attributes in each\nschema.\n\n```elixir\ndefmodule MyApp.Event do\n  use MyApp.Schema\n\n  schema \"events\" do\n    # additional fields\n  end\nend\n```\n\n#","ref":"readme.html#shared-schema-configuration"},{"type":"extras","title":"Manual ULID Generation - Ecto.ULID Next","doc":"##","ref":"readme.html#manual-ulid-generation"},{"type":"extras","title":"Using the Current System Time - Ecto.ULID Next","doc":"Use `Ecto.ULID.generate/0` or `Ecto.ULID.bingenerate/0` to create ULIDs in\nstring or binary format based on the current system time. This is useful for\nsending ULIDs to external systems or for use with `c:Ecto.Repo.insert_all/3`,\nwhere autogenerated fields are not set automatically.\n\n```elixir\niex> Ecto.ULID.generate()\n\"01BZ13RV29T5S8HV45EDNC748P\"\n\niex> Ecto.ULID.bingenerate()\n >\n```\n\n##","ref":"readme.html#using-the-current-system-time"},{"type":"extras","title":"Specifying a Timestamp - Ecto.ULID Next","doc":"To generate a ULID based on a specific timestamp—useful for backfilling\nhistorical data—you can use `Ecto.ULID.generate/1` or `Ecto.ULID.bingenerate/1`.\n\n```elixir\niex> Ecto.ULID.generate(1402899630000)\n\"018THNB1XGQZ7T929PD126SM3Z\"\n\niex> Ecto.ULID.bingenerate(1402899630000)\n >\n```","ref":"readme.html#specifying-a-timestamp"},{"type":"extras","title":"Further Reading - Ecto.ULID Next","doc":"#","ref":"readme.html#further-reading"},{"type":"extras","title":"Specifications - Ecto.ULID Next","doc":"- [ULID specification](https://github.com/ulid/spec)\n- [UUID v7 specification](https://datatracker.ietf.org/doc/html/draft-ietf-uuidrev-rfc4122bis#name-uuid-version-7)\n\n#","ref":"readme.html#specifications"},{"type":"extras","title":"Database Support and Tools - Ecto.ULID Next","doc":"ULIDs are not natively supported by most databases, which means querying data\ndirectly may display IDs in UUID format rather than the Crockford Base32\nrepresentation. For PostgreSQL, the following tool can help convert between ULID\nand UUID formats:\n\n- [pgsql-ulid](https://github.com/scoville/pgsql-ulid)\n\n#","ref":"readme.html#database-support-and-tools"},{"type":"extras","title":"Articles and Insights - Ecto.ULID Next","doc":"- [Understanding UUIDs, ULIDs and String Representations](https://sudhir.io/uuids-ulids)","ref":"readme.html#articles-and-insights"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"## [1.0.2] - 2023-09-09\n\n#","ref":"changelog.html#unreleased"},{"type":"extras","title":"Changed - Changelog","doc":"- Documentation improvements.\n\n## [1.0.1] - 2022-12-14\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Source references in documentation.\n\n## [1.0.0] - 2022-12-14\n\nStable release.\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Minor dependency and documentation updates.\n\n## [0.3.1] - 2022-02-22\n\nMaintenance release\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Elixir version support now follows the\n  [officially supported Elixir versions](https://hexdocs.pm/elixir/1.13/compatibility-and-deprecations.html).\n- Update test matrix accordingly.\n- Improve documentation and type specs.\n\n## [0.3.0] - 2020-11-16\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Minimum supported Ecto is now 3.2.\n- Minimum supported Elixir is now 1.7.\n\n## [0.2.0] - 2019-01-17\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Minimum supported Elixir is now 1.4.\n- ([#3](https://github.com/TheRealReal/ecto-ulid/pull/3))\n  Fix deprecation warnings regarding time units.\n\n## [0.1.1] - 2018-12-03\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- ([#2](https://github.com/TheRealReal/ecto-ulid/pull/2))\n  Add support for Ecto 3.x.\n\n## [0.1.0] - 2018-06-06\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Generate ULID in Base32 or binary format.\n- Generate ULID for a given timestamp.\n- Autogenerate ULID when used as a primary key.\n- Supports reading and writing ULID in a database backed by its native `uuid` type (no database\n  extensions required).\n- Supports Ecto 2.x.\n- Supports Elixir 1.2 and newer.\n- Tested with PostgreSQL and MySQL.","ref":"changelog.html#added"}],"content_type":"text/markdown"}